<?php

/**
 * Handler for displaying an entire Sarnia entity as a Views field.
 */
class SarniaViewsHandlerFieldEntity extends views_handler_field {

  /**
   * A Sarnia entity bundle info array.
   * @var array
   */
  protected $sarnia_type_info;

  /**
   * Initialize the handler and load Sarnia entity type info.
   */
  function init(&$view, &$options) {
    parent::init($view, $options);
    $this->sarnia_type_info = sarnia_entity_type_load_by_index($this->definition['search_api_index']);
  }

  /**
   * Specify handler options, adding formatter-specific properties.
   */
  public function option_definition() {
    $options = parent::option_definition();
    $options['view mode'] = array('default' => 'default');
    return $options;
  }

  /**
   * Add a dropdown to select an entity view mode to the handler configuration form.
   */
  public function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $view_modes = array();
    $entity_info = entity_get_info($this->sarnia_type_info['machine_name']);
    foreach ($entity_info['view modes'] as $view_mode => $view_mode_info) {
      $view_modes[$view_mode] = $view_mode_info['label'];
    }
    if (empty($view_modes)) {
      $view_modes['default'] = t('Default');
    }
    $form['view_mode'] = array(
      '#type' => 'select',
      '#title' => t('View mode'),
      '#options' => $view_modes,
      '#default_value' => $this->options['view mode'],
    );
  }

  /**
   * For Sarnia, this handler's "real field" is 'entity', which Search API
   * provides by default. We don't have to do anything.
   */
  public function query() {
    $this->field_alias = $this->real_field;
  }

  /**
   * Load any entities that aren't present.
   */
  function pre_render(&$results) {
    // Get a list of unloaded entities.
    $ids = array();
    foreach ($results as $result_id => $result) {
      $entity_id = &$result->{$this->field_alias};
      if (!is_object($entity_id)) {
        $ids[$result_id] = $entity_id;
      }
    }
    // Load whatever unloaded entities we found.
    if (!empty($ids)) {
      $entities = entity_load($this->sarnia_type_info['machine_name'], array_values($ids));
      foreach ($ids as $result_id => $entity_id) {
        if ($entities[$entity_id]) {
          $results[$result_id]->{$this->field_alias} = $entities[$entity_id];
        }
      }
    }
  }

  /**
   * Render a field.
   */
  function render($values) {
    $entity = $this->get_value($values);
    if (is_object($entity)) {
      return sarnia_view($entity, $this->options['view mode']);
    }
  }

  /**
   * Get the value (a full Sarnia entity) to render.
   *
   * @param $values
   *   An object containing all retrieved values.
   * @param $field
   *   Optional name of the field where the value is stored.
   */
  public function _get_value($values, $field = NULL) {}

  /**
   * Display the view mode in the admin interface summary.
   */
  public function admin_summary() {
    $summary = parent::admin_summary();
    if ($this->options['view mode'] != 'default') {
      $summary .= t(', view mode = @mode', array('@mode' => $this->options['view mode']));
    }
    return $summary;
  }

}
